@@ -5,7 +5,7 @@ use gleam_core::{
     build::{Codegen, Compile, Mode, Options, Package, Target},
     config::{PackageConfig, SpdxLicense},
     docs::DocContext,
-    error::SmallVersion,
+    error::{wrap, SmallVersion},
     hex,
     paths::{self, ProjectPaths},
     requirement::Requirement,
@@ -16,83 +16,108 @@ use itertools::Itertools;
 use sha2::Digest;
 use std::{io::Write, path::PathBuf, time::Instant};
 
-use crate::{build, cli, docs, fs, hex::ApiKeyCommand, http::HttpClient};
+use crate::{build, cli, docs, fs, http::HttpClient};
 
-pub fn command(replace: bool, yes: bool) -> Result<()> {
-    let command = PublishCommand::setup(replace, yes)?;
+pub fn command(replace: bool, i_am_sure: bool) -> Result<()> {
+    let paths = crate::find_project_paths()?;
+    let mut config = crate::config::root_config()?;
 
-    if let Some(mut command) = command {
-        command.run()?;
+    let should_publish = check_for_gleam_prefix(&config)?
+        && check_for_version_zero(&config)?
+        && check_repo_url(&config, i_am_sure)?;
+
+    if !should_publish {
+        println!("Not publishing.");
+        return Ok(());
+    }
+
+    let Tarball {
+        mut compile_result,
+        data: package_tarball,
+        src_files_added,
+        generated_files_added,
+    } = do_build_hex_tarball(&paths, &mut config)?;
+
+    check_for_name_squatting(&compile_result)?;
+    check_for_multiple_top_level_modules(&compile_result, i_am_sure)?;
+
+    // Build HTML documentation
+    let docs_tarball = fs::create_tar_archive(docs::build_documentation(
+        &config,
+        &mut compile_result,
+        DocContext::HexPublish,
+    )?)?;
+
+    // Ask user if this is correct
+    if !generated_files_added.is_empty() {
+        println!("\nGenerated files:");
+        for file in generated_files_added.iter().sorted() {
+            println!("  - {}", file.0);
+        }
+    }
+    println!("\nSource files:");
+    for file in src_files_added.iter().sorted() {
+        println!("  - {file}");
+    }
+    println!("\nName: {}", config.name);
+    println!("Version: {}", config.version);
+
+    let should_publish = i_am_sure || cli::confirm("\nDo you wish to publish this package?")?;
+    if !should_publish {
+        println!("Not publishing.");
+        return Ok(());
+    }
+
+    let runtime = tokio::runtime::Runtime::new().expect("Unable to start Tokio async runtime");
+    let hex_config = hexpm::Config::new();
+    let api_key =
+        crate::hex::HexAuthentication::new(&runtime, hex_config.clone()).get_or_create_api_key()?;
+    let start = Instant::now();
+    cli::print_publishing(&config.name, &config.version);
+
+    runtime.block_on(hex::publish_package(
+        package_tarball,
+        config.version.to_string(),
+        &api_key,
+        &hex_config,
+        replace,
+        &HttpClient::new(),
+    ))?;
+
+    cli::print_publishing_documentation();
+    runtime.block_on(hex::publish_documentation(
+        &config.name,
+        &config.version,
+        docs_tarball,
+        &api_key,
+        &hex_config,
+        &HttpClient::new(),
+    ))?;
+    cli::print_published(start.elapsed());
+    println!(
+        "\nView your package at https://hex.pm/packages/{}",
+        &config.name
+    );
+
+    // Prompt the user to make a git tag if they have not.
+    let has_repo = config.repository.url().is_some();
+    let git = PathBuf::from(".git");
+    let version = format!("v{}", &config.version);
+    let git_tag = git.join("refs").join("tags").join(&version);
+    if has_repo && git.exists() && !git_tag.exists() {
+        println!(
+            "
+Please push a git tag for this release so source code links in the
+HTML documentation will work:
+
+    git tag {version}
+    git push origin {version}
+"
+        )
     }
     Ok(())
 }
 
-pub struct PublishCommand {
-    config: PackageConfig,
-    package_tarball: Vec<u8>,
-    docs_tarball: Vec<u8>,
-    replace: bool,
-}
-
-impl PublishCommand {
-    pub fn setup(replace: bool, i_am_sure: bool) -> Result<Option<Self>> {
-        let paths = crate::find_project_paths()?;
-        let mut config = crate::config::root_config()?;
-
-        let should_publish = check_for_gleam_prefix(&config, i_am_sure)?
-            && check_for_version_zero(&config, i_am_sure)?
-            && check_repo_url(&config, i_am_sure)?;
-
-        if !should_publish {
-            println!("Not publishing.");
-            std::process::exit(0);
-        }
-
-        let Tarball {
-            mut compile_result,
-            data: package_tarball,
-            src_files_added,
-            generated_files_added,
-        } = do_build_hex_tarball(&paths, &mut config)?;
-
-        check_for_name_squatting(&compile_result)?;
-
-        // Build HTML documentation
-        let docs_tarball = fs::create_tar_archive(docs::build_documentation(
-            &config,
-            &mut compile_result,
-            DocContext::HexPublish,
-        )?)?;
-
-        // Ask user if this is correct
-        if !generated_files_added.is_empty() {
-            println!("\nGenerated files:");
-            for file in generated_files_added.iter().sorted() {
-                println!("  - {}", file.0);
-            }
-        }
-        println!("\nSource files:");
-        for file in src_files_added.iter().sorted() {
-            println!("  - {file}");
-        }
-        println!("\nName: {}", config.name);
-        println!("Version: {}", config.version);
-
-        let should_publish = i_am_sure || cli::confirm("\nDo you wish to publish this package?")?;
-        if !should_publish {
-            println!("Not publishing.");
-            std::process::exit(0);
-        }
-
-        Ok(Some(Self {
-            config,
-            docs_tarball,
-            package_tarball,
-            replace,
-        }))
-    }
-}
-
 fn check_for_name_squatting(package: &Package) -> Result<(), Error> {
     if package.modules.len() > 1 {
         return Ok(());
@@ -123,6 +148,50 @@ fn check_for_name_squatting(package: &Package) -> Result<(), Error> {
     Ok(())
 }
 
+fn check_for_multiple_top_level_modules(package: &Package, i_am_sure: bool) -> Result<(), Error> {
+    // Collect top-level module names
+    let mut top_level_module_names = package
+        .modules
+        .iter()
+        .filter_map(|module| module.name.split('/').next())
+        .collect::<Vec<_>>();
+
+    // Remove duplicates
+    top_level_module_names.sort_unstable();
+    top_level_module_names.dedup();
+
+    // If more than one top-level module name is found, prompt for confirmation
+    if top_level_module_names.len() > 1 {
+        let text = wrap(&format!(
+            "Your package defines multiple top-level modules: {}.
+
+Defining multiple top-level modules can lead to namespace pollution \
+and potential conflicts for consumers.
+
+To fix this, move all your modules under a single top-level module of your choice.
+
+For example:
+  src/{1}.gleam
+  src/{1}/module1.gleam
+  src/{1}/module2.gleam",
+            top_level_module_names.join(", "),
+            package.config.name
+        ));
+        println!("{text}\n");
+
+        let should_publish =
+            i_am_sure || cli::confirm("\nDo you wish to continue publishing this package?")?;
+        println!();
+
+        if !should_publish {
+            println!("Not publishing.");
+            std::process::exit(0);
+        }
+    }
+
+    Ok(())
+}
+
 fn check_repo_url(config: &PackageConfig, i_am_sure: bool) -> Result<bool, Error> {
     let Some(url) = config.repository.url() else {
         return Ok(true);
@@ -147,7 +216,7 @@ valid, {} returned status {}",
 }
 
 /// Ask for confirmation if the package name if a v0.x.x version
-fn check_for_version_zero(config: &PackageConfig, i_am_sure: bool) -> Result<bool, Error> {
+fn check_for_version_zero(config: &PackageConfig) -> Result<bool, Error> {
     if config.version.major != 0 {
         return Ok(true);
     }
@@ -158,15 +227,19 @@ fn check_for_version_zero(config: &PackageConfig, i_am_sure: bool) -> Result<boo
 Semantic versioning doesn't apply to version 0.x.x releases, so your
 users will not be protected from breaking changes. This can result
 in a poor user experience where packages can break unexpectedly with
-updates that would normally be safe."
+updates that would normally be safe.
+
+If your package is not ready to be used in production it should not
+be published.
+\n"
     );
-    let should_publish = i_am_sure || cli::confirm("\nDo you wish to continue?")?;
+    let should_publish = cli::confirm_with_text("I am not using semantic versioning")?;
     println!();
     Ok(should_publish)
 }
 
 /// Ask for confirmation if the package name if `gleam_*`
-fn check_for_gleam_prefix(config: &PackageConfig, i_am_sure: bool) -> Result<bool, Error> {
+fn check_for_gleam_prefix(config: &PackageConfig) -> Result<bool, Error> {
     if !config.name.starts_with("gleam_") || config.name.starts_with("gleam_community_") {
         return Ok(true);
     }
@@ -174,68 +247,13 @@ fn check_for_gleam_prefix(config: &PackageConfig, i_am_sure: bool) -> Result<boo
     println!(
         "You are about to publish a package with a name that starts with
 the prefix `gleam_`, which is for packages maintained by the Gleam
-core team.",
+core team.\n",
     );
-    let should_publish =
-        i_am_sure || cli::confirm("\nAre you sure you want to use this package name?")?;
+    let should_publish = cli::confirm_with_text("I am part of the Gleam core team")?;
     println!();
     Ok(should_publish)
 }
 
-impl ApiKeyCommand for PublishCommand {
-    fn with_api_key(
-        &mut self,
-        runtime: &tokio::runtime::Handle,
-        hex_config: &hexpm::Config,
-        api_key: &str,
-    ) -> Result<()> {
-        let start = Instant::now();
-        cli::print_publishing(&self.config.name, &self.config.version);
-
-        runtime.block_on(hex::publish_package(
-            std::mem::take(&mut self.package_tarball),
-            self.config.version.to_string(),
-            api_key,
-            hex_config,
-            self.replace,
-            &HttpClient::new(),
-        ))?;
-
-        cli::print_publishing_documentation();
-        runtime.block_on(hex::publish_documentation(
-            &self.config.name,
-            &self.config.version,
-            std::mem::take(&mut self.docs_tarball),
-            api_key,
-            hex_config,
-            &HttpClient::new(),
-        ))?;
-        cli::print_published(start.elapsed());
-        println!(
-            "\nView your package at https://hex.pm/packages/{}",
-            &self.config.name
-        );
-
-        // Prompt the user to make a git tag if they have not.
-        let has_repo = self.config.repository.url().is_some();
-        let git = PathBuf::from(".git");
-        let version = format!("v{}", &self.config.version);
-        let git_tag = git.join("refs").join("tags").join(&version);
-        if has_repo && git.exists() && !git_tag.exists() {
-            println!(
-                "
-Please push a git tag for this release so source code links in the
-HTML documentation will work:
-
-    git tag {version}
-    git push origin {version}
-"
-            )
-        }
-        Ok(())
-    }
-}
-
 struct Tarball {
     compile_result: Package,
     data: Vec<u8>,
@@ -732,13 +750,6 @@ fn prevent_publish_git_dependency() {
     );
 }
 
-pub fn get_hostname() -> String {
-    hostname::get()
-        .expect("Looking up hostname")
-        .to_string_lossy()
-        .to_string()
-}
-
 fn quotes(x: &str) -> String {
     format!(r#"<<"{x}">>"#)
 }
