@@ -141,7 +141,7 @@ pub struct ModuleAnalyzerConstructor<'a, A> {
     pub package_config: &'a PackageConfig,
 }
 
-impl<'a, A> ModuleAnalyzerConstructor<'a, A> {
+impl<A> ModuleAnalyzerConstructor<'_, A> {
     /// Crawl the AST, annotating each node with the inferred type or
     /// returning an error.
     ///
@@ -823,7 +823,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             } => self.track_feature_usage(FeatureKind::InternalAnnotation, location),
         }
 
-        let constructors = constructors
+        let constructors: Vec<RecordConstructor<Arc<Type>>> = constructors
             .into_iter()
             .map(
                 |RecordConstructor {
@@ -832,8 +832,15 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                      name,
                      arguments: args,
                      documentation,
+                     deprecation: constructor_deprecation,
                  }| {
                     self.check_name_case(name_location, &name, Named::CustomTypeVariant);
+                    if constructor_deprecation.is_deprecated() {
+                        self.track_feature_usage(
+                            FeatureKind::VariantWithDeprecatedAnnotation,
+                            location,
+                        );
+                    }
 
                     let preregistered_fn = environment
                         .get_variable(&name)
@@ -868,6 +875,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                         name,
                         arguments: args,
                         documentation,
+                        deprecation: constructor_deprecation,
                     }
                 },
             )
@@ -878,6 +886,36 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
             .parameters
             .clone();
 
+        // Check if all constructors are deprecated if so error.
+        if !constructors.is_empty()
+            && constructors
+                .iter()
+                .all(|record| record.deprecation.is_deprecated())
+        {
+            self.problems
+                .error(Error::AllVariantsDeprecated { location });
+        }
+
+        // If any constructor record/varient is deprecated while
+        // the type is deprecated as a whole that is considered an error.
+        if deprecation.is_deprecated()
+            && !constructors.is_empty()
+            && constructors
+                .iter()
+                .any(|record| record.deprecation.is_deprecated())
+        {
+            // Report error on all variants attibuted with deprecated
+            constructors
+                .iter()
+                .filter(|record| record.deprecation.is_deprecated())
+                .for_each(|record| {
+                    self.problems
+                        .error(Error::DeprecatedVariantOnDeprecatedType {
+                            location: record.location,
+                        });
+                });
+        }
+
         Ok(Definition::CustomType(CustomType {
             documentation: doc,
             location,
@@ -1014,11 +1052,19 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 *publicity
             };
 
+            // If the whole custom type is deprecated all of its varints are too.
+            // Otherwise just the varint(s) attributed as deprecated are.
+            let deprecate_constructor = if deprecation.is_deprecated() {
+                deprecation
+            } else {
+                &constructor.deprecation
+            };
+
             environment.insert_module_value(
                 constructor.name.clone(),
                 ValueConstructor {
                     publicity: value_constructor_publicity,
-                    deprecation: deprecation.clone(),
+                    deprecation: deprecate_constructor.clone(),
                     type_: type_.clone(),
                     variant: constructor_info.clone(),
                 },
@@ -1042,7 +1088,7 @@ impl<'a, A> ModuleAnalyzer<'a, A> {
                 constructor_info,
                 type_,
                 value_constructor_publicity,
-                deprecation.clone(),
+                deprecate_constructor.clone(),
             );
 
             environment.names.named_constructor_in_scope(
@@ -1762,10 +1808,10 @@ fn get_compatible_record_fields<A: std::fmt::Debug>(
             };
 
             // The labels must be the same
-            if !argument
+            if argument
                 .label
                 .as_ref()
-                .is_some_and(|(_, arg_label)| arg_label == first_label)
+                .is_none_or(|(_, arg_label)| arg_label != first_label)
             {
                 continue 'next_argument;
             }
