@@ -1,5 +1,7 @@
 use std::sync::OnceLock;
 
+use type_::{FieldMap, TypedCallArg};
+
 use super::*;
 use crate::type_::{bool, HasType, Type, ValueConstructorVariant};
 
@@ -133,11 +135,21 @@ pub enum TypedExpr {
         segments: Vec<TypedExprBitArraySegment>,
     },
 
+    /// A record update gets desugared to a block expression of the form
+    ///
+    /// {
+    ///   let _record = record
+    ///   Constructor(explicit_arg: explicit_value(), implicit_arg: _record.implicit_arg)
+    /// }
+    ///
+    /// We still keep a separate `RecordUpdate` AST node for the same reasons as
+    /// we do for pipelines.
     RecordUpdate {
         location: SrcSpan,
         type_: Arc<Type>,
-        record: Box<Self>,
-        args: Vec<TypedRecordUpdateArg>,
+        record: TypedAssignment,
+        constructor: Box<Self>,
+        args: Vec<CallArg<Self>>,
     },
 
     NegateBool {
@@ -188,8 +200,7 @@ impl TypedExpr {
                 // We don't want to match on todos that were implicitly inserted
                 // by the compiler as it would result in confusing suggestions
                 // from the LSP.
-                TodoKind::EmptyFunction => None,
-                TodoKind::IncompleteUse => None,
+                TodoKind::EmptyFunction | TodoKind::EmptyBlock | TodoKind::IncompleteUse => None,
             },
 
             Self::Pipeline {
@@ -303,6 +314,7 @@ impl TypedExpr {
 
             Self::RecordUpdate { record, args, .. } => args
                 .iter()
+                .filter(|arg| arg.implicit.is_none())
                 .find_map(|arg| arg.find_node(byte_index))
                 .or_else(|| record.find_node(byte_index))
                 .or_else(|| self.self_if_contains_location(byte_index)),
@@ -549,19 +561,45 @@ impl TypedExpr {
             // long as it's not called!
             TypedExpr::ModuleSelect { .. } => true,
 
-            // A pipeline is a pure value constructor if its last step is a record builder.
-            // For example `wibble() |> wobble() |> Ok`
-            TypedExpr::Pipeline { finally, .. } => finally.is_record_builder(),
+            // A pipeline is a pure value constructor if its last step is a record builder,
+            // or a call to a fn expression that has a body comprised of just pure value
+            // constructors. For example:
+            //  - `wibble() |> wobble() |> Ok`
+            //  - `"hello" |> fn(s) { s <> " world!" }`
+            TypedExpr::Pipeline { finally, .. } => match finally.as_ref() {
+                TypedExpr::Fn { body, .. } => body.iter().all(|s| s.is_pure_value_constructor()),
+                fun => fun.is_pure_value_constructor(),
+            },
 
-            // A function call is a pure record constructor if it is a record builder.
-            // For example `Ok(wobble(wibble()))`
-            TypedExpr::Call { fun, .. } => fun.as_ref().is_record_builder(),
+            TypedExpr::Call { fun, .. } => match fun.as_ref() {
+                // Immediately calling a fn expression that has a body comprised of just
+                // pure value constructors is in itself pure.
+                TypedExpr::Fn { body, .. } => body.iter().all(|s| s.is_pure_value_constructor()),
+                // And calling a record builder is a pure value constructor:
+                // `Some(1)`
+                fun => fun.is_record_builder(),
+            },
 
-            // Blocks and Cases are not considered pure value constructors for now,
-            // in the future we might want to do something a bit smarter and inspect
-            // their content to see if they end up returning something that is a
-            // pure value constructor and raise a warning for those as well.
-            TypedExpr::Block { .. } | TypedExpr::Case { .. } => false,
+            // A block is pure if all the statements it's made of are pure.
+            // For example `{ True 1 }`
+            TypedExpr::Block { statements, .. } => {
+                statements.iter().all(|s| s.is_pure_value_constructor())
+            }
+
+            // A case is pure if its subject and all its branches are.
+            // For example:
+            // ```gleam
+            // case 1 + 1 {
+            //   0 -> 1
+            //   _ -> 2
+            // }
+            // ```
+            TypedExpr::Case {
+                subjects, clauses, ..
+            } => {
+                subjects.iter().all(|s| s.is_pure_value_constructor())
+                    && clauses.iter().all(|c| c.then.is_pure_value_constructor())
+            }
 
             // `panic`, `todo`, and placeholders are never considered pure value constructors,
             // we don't want to raise a warning for an unused value if it's one
@@ -570,6 +608,7 @@ impl TypedExpr {
         }
     }
 
+    #[must_use]
     pub fn is_record_builder(&self) -> bool {
         match self {
             TypedExpr::Call { fun, .. } => fun.is_record_builder(),
@@ -607,6 +646,82 @@ impl TypedExpr {
             _ => false,
         }
     }
+
+    pub(crate) fn call_arguments(&self) -> Option<&Vec<TypedCallArg>> {
+        match self {
+            TypedExpr::Call { args, .. } => Some(args),
+            _ => None,
+        }
+    }
+
+    pub(crate) fn fn_expression_body(&self) -> Option<&Vec1<TypedStatement>> {
+        match self {
+            TypedExpr::Fn { body, .. } => Some(body),
+            _ => None,
+        }
+    }
+
+    // If the expression is a fn or a block then returns the location of its
+    // last element, otherwise it returns the location of the whole expression.
+    pub fn last_location(&self) -> SrcSpan {
+        match self {
+            TypedExpr::Int { location, .. }
+            | TypedExpr::Float { location, .. }
+            | TypedExpr::String { location, .. }
+            | TypedExpr::Var { location, .. }
+            | TypedExpr::List { location, .. }
+            | TypedExpr::Call { location, .. }
+            | TypedExpr::BinOp { location, .. }
+            | TypedExpr::Case { location, .. }
+            | TypedExpr::RecordAccess { location, .. }
+            | TypedExpr::ModuleSelect { location, .. }
+            | TypedExpr::Tuple { location, .. }
+            | TypedExpr::TupleIndex { location, .. }
+            | TypedExpr::Todo { location, .. }
+            | TypedExpr::Panic { location, .. }
+            | TypedExpr::BitArray { location, .. }
+            | TypedExpr::RecordUpdate { location, .. }
+            | TypedExpr::NegateBool { location, .. }
+            | TypedExpr::NegateInt { location, .. }
+            | TypedExpr::Invalid { location, .. }
+            | TypedExpr::Pipeline { location, .. } => *location,
+
+            TypedExpr::Block { statements, .. } => statements.last().last_location(),
+            TypedExpr::Fn { body, .. } => body.last().last_location(),
+        }
+    }
+
+    pub fn field_map(&self) -> Option<&FieldMap> {
+        match self {
+            TypedExpr::Int { .. }
+            | TypedExpr::Float { .. }
+            | TypedExpr::String { .. }
+            | TypedExpr::Block { .. }
+            | TypedExpr::Pipeline { .. }
+            | TypedExpr::Fn { .. }
+            | TypedExpr::List { .. }
+            | TypedExpr::Call { .. }
+            | TypedExpr::BinOp { .. }
+            | TypedExpr::Case { .. }
+            | TypedExpr::RecordAccess { .. }
+            | TypedExpr::Tuple { .. }
+            | TypedExpr::TupleIndex { .. }
+            | TypedExpr::Todo { .. }
+            | TypedExpr::Panic { .. }
+            | TypedExpr::BitArray { .. }
+            | TypedExpr::RecordUpdate { .. }
+            | TypedExpr::NegateBool { .. }
+            | TypedExpr::NegateInt { .. }
+            | TypedExpr::Invalid { .. } => None,
+
+            TypedExpr::Var { constructor, .. } => constructor.field_map(),
+            TypedExpr::ModuleSelect { constructor, .. } => match constructor {
+                ModuleValueConstructor::Record { field_map, .. }
+                | ModuleValueConstructor::Fn { field_map, .. } => field_map.as_ref(),
+                ModuleValueConstructor::Constant { .. } => None,
+            },
+        }
+    }
 }
 
 impl<'a> From<&'a TypedExpr> for Located<'a> {
