@@ -6,6 +6,7 @@ mod bit_array;
 mod blocks;
 mod cases;
 mod conditional_compilation;
+mod custom_type;
 mod external_fn;
 mod external_types;
 mod function;
@@ -257,85 +258,6 @@ type Many(a) =
     );
 }
 
-#[test]
-fn custom_types() {
-    assert_format!(
-        "type WowThisTypeHasJustTheLongestName(
-  some_long_type_variable,
-  and_another,
-  and_another_again,
-) {
-  Make
-}
-"
-    );
-
-    assert_format!(
-        "type Result(a, e) {
-  Ok(a)
-  Error(e)
-}
-"
-    );
-
-    assert_format!(
-        "type Result(a, e) {
-  Ok(value: a)
-  Error(error: e)
-}
-"
-    );
-
-    assert_format!(
-        "type SillyResult(a, e) {
-  Ok(
-    first_value_with_really_long_name: a,
-    second_value_with_really_long_name: a,
-  )
-  Error(error: e)
-}
-"
-    );
-
-    assert_format!(
-        "type SillyResult(a, e) {
-  Ok(
-    first_value_with_really_long_name: a,
-    second_value_with_really_long_name: List(
-      #(Int, fn(a, a, a, a, a, a, a) -> List(a)),
-    ),
-  )
-  Error(error: e)
-}
-"
-    );
-
-    assert_format!(
-        "type X {
-  X(
-    start: fn() -> a_reall_really_long_name_goes_here,
-    stop: fn() -> a_reall_really_long_name_goes_here,
-  )
-}
-"
-    );
-
-    assert_format!(
-        "pub opaque type X {
-  X
-}
-"
-    );
-
-    assert_format!(
-        "///
-pub type Option(a) {
-  None
-}
-"
-    );
-}
-
 #[test]
 fn expr_fn() {
     assert_format!(
@@ -3325,49 +3247,6 @@ type Whatever =
     );
 }
 
-#[test]
-fn doc_comments_7_test() {
-    assert_format!(
-        r#"import one
-
-/// one
-///two
-type Whatever {
-  Whatever
-}
-"#
-    );
-}
-
-#[test]
-fn comments1() {
-    assert_format!(
-        r#"import one
-
-// one
-//two
-type Whatever {
-  Whatever
-}
-"#
-    );
-}
-
-#[test]
-fn comments2() {
-    assert_format!(
-        r#"import one
-
-// one
-//two
-/// three
-type Whatever {
-  Whatever
-}
-"#
-    );
-}
-
 #[test]
 fn comments3() {
     assert_format!(
@@ -3402,38 +3281,6 @@ fn whatever() -> Nil
     );
 }
 
-#[test]
-fn comments6() {
-    assert_format!(
-        r#"// one
-//two
-type Thingy
-"#
-    );
-}
-
-#[test]
-fn comments7() {
-    assert_format!(
-        r#"// one
-//two
-type Thingy
-"#
-    );
-}
-
-#[test]
-fn comments8() {
-    assert_format!(
-        r#"// one
-//two
-type Whatever {
-  Whatever
-}
-"#
-    );
-}
-
 #[test]
 fn comments9() {
     assert_format!(
@@ -3445,29 +3292,6 @@ type Whatever =
     );
 }
 
-#[test]
-fn comments10() {
-    assert_format!(
-        r#"// zero
-import one
-
-// one
-//two
-type Whatever {
-  Whatever
-}
-"#
-    );
-
-    assert_format!(
-        "fn main() {
-  // Hello
-  \"world\"
-}
-"
-    );
-}
-
 #[test]
 fn comment23() {
     assert_format!(
@@ -4936,24 +4760,6 @@ pub fn main() {
     );
 }
 
-// https://github.com/gleam-lang/gleam/issues/1757
-#[test]
-fn multiple_line_custom_type_constructor_field_doc_comments() {
-    assert_format!(
-        r#"pub type Thingy {
-  Thingy(
-    /// One?
-    /// One!
-    one: One,
-    /// Two?
-    /// Two!
-    two: Two,
-  )
-}
-"#
-    );
-}
-
 // https://github.com/gleam-lang/gleam/issues/1872
 #[test]
 fn multiple_line_spread_list_comments() {
@@ -5361,17 +5167,6 @@ fn comment_at_end_of_type() {
     );
 }
 
-#[test]
-fn deprecated_custom_type() {
-    assert_format!(
-        r#"@deprecated("Deprecated type")
-pub type One {
-  One
-}
-"#
-    );
-}
-
 #[test]
 fn deprecated_type_alias() {
     assert_format!(
@@ -6457,3 +6252,143 @@ fn record_inside_const_list() {
 "#
     );
 }
+
+#[test]
+fn formatter_adds_todo_inside_empty_block() {
+    assert_format_rewrite!(
+        "pub fn main() {{}}",
+        r#"pub fn main() {
+  { todo }
+}
+"#
+    );
+}
+
+#[test]
+fn let_assert_as() {
+    assert_format!(
+        r#"pub fn main() {
+  let assert 10 = 10 as "10 == 10"
+}
+"#
+    );
+}
+
+#[test]
+fn let_assert_as_long_message() {
+    assert_format!(
+        r#"pub fn main() {
+  let assert Ok(10) = Ok(10)
+    as "It's pretty obvious that this will never fail, but just in case, here is why."
+}
+"#
+    );
+}
+
+#[test]
+fn let_assert_as_long_message_and_value() {
+    assert_format!(
+        r#"pub fn main() {
+  let assert Ok(something) =
+    some_very_long_variable_which_always_represents_a_successful_result
+    as "As you can see by the incredibly descriptive variable name, this operation never fails."
+}
+"#
+    );
+}
+
+#[test]
+fn let_assert_as_concatenated_message() {
+    assert_format!(
+        r#"pub fn main() {
+  let assert 1 = 2 as { "This will" <> " " <> "crash" }
+}
+"#
+    );
+}
+
+#[test]
+fn let_assert_as_variable_message() {
+    assert_format!(
+        r#"pub fn main() {
+  let message = "Hi :)"
+  let assert 1 = 2 as message
+}
+"#
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/4121
+#[test]
+fn function_capture_formatted_like_regular_calls() {
+    assert_format!(
+        r#"pub fn main() {
+  capture(a, _, [
+    really_long_thing_that_can_be_broken,
+    something_else_for_good_measure,
+  ])
+}
+"#
+    );
+}
+
+#[test]
+fn function_capture_formatted_like_regular_calls_2() {
+    assert_format!(
+        r#"pub fn main() {
+  capture(
+    a,
+    _,
+    really_long_thing_that_cannot_be_broken,
+    something_else_for_good_measure,
+  )
+}
+"#
+    );
+}
+
+#[test]
+fn function_capture_formatted_like_regular_calls_3() {
+    assert_format!(
+        r#"pub fn main() {
+  list.fold(my_list, _, fn(a) {
+    io.print("Meh")
+    io.print("Meh")
+  })
+}
+"#
+    );
+}
+
+#[test]
+fn function_capture_formatted_like_regular_calls_inside_a_long_list() {
+    assert_format!(
+        r#"pub fn main() {
+  [
+    capture(a, _, [
+      really_long_thing_that_can_be_broken,
+      something_else_for_good_measure,
+    ]),
+    regular_call(a, [
+      really_long_thing_that_can_be_broken,
+      something_else_for_good_measure,
+    ]),
+  ]
+}
+"#
+    );
+}
+
+#[test]
+fn function_capture_formatted_like_regular_calls_in_a_pipe() {
+    assert_format!(
+        r#"pub fn main() {
+  [1, 2, 3]
+  |> list.fold(from: 1, over: _, with: fn(a, b) {
+    // a comment!
+    a + b
+  })
+}
+"#
+    );
+}
