@@ -60,3 +60,22 @@ fn a_comment_before_a_guard_doesnt_force_it_to_break() {
 "#
     );
 }
+
+#[test]
+fn long_guard_with_alternative_patterns() {
+    assert_format!(
+        r#"pub fn main() {
+  case wibble {
+    Wibble(first_one)
+      | Wibble(another_one)
+      | Wibble(
+        this_is_extra_long_to_go_over_the_line_limit,
+        this_gets_broken_as_well,
+      )
+      if True
+    -> Ok(wibble)
+  }
+}
+"#
+    );
+}
