@@ -1,3 +1,4 @@
+use num_bigint::BigInt;
 use std::sync::OnceLock;
 
 use super::{expression::is_js_scalar, *};
@@ -518,7 +519,16 @@ impl<'module_ctx, 'expression_gen, 'a> Generator<'module_ctx, 'expression_gen, '
                     // let prefix = "wibble";
                     // ^^^^^^^^^^^^^^^^^^^^^ we're adding this assignment inside the if clause
                     //                       the case branch gets translated into.
-                    self.push_assignment(expression::string(left_side_string), left);
+                    //
+                    // We also want to push this assignment without using push_assignment, since we
+                    // do _not_ want to access the current path on the static string!
+                    let var = self.next_local_var(left);
+                    self.assignments.push(Assignment {
+                        subject: expression::string(left_side_string),
+                        path: docvec![],
+                        name: left,
+                        var,
+                    });
                 }
                 Ok(())
             }
@@ -580,28 +590,49 @@ impl<'module_ctx, 'expression_gen, 'a> Generator<'module_ctx, 'expression_gen, '
                     {
                         let details = Self::sized_bit_array_segment_details(segment)?;
 
-                        let start = offset.bytes;
-                        let increment = details.size / 8;
-                        let end = offset.bytes + increment;
-
-                        if segment.type_ == crate::type_::int() {
-                            if details.size == 8 && !details.is_signed {
-                                self.push_byte_at(offset.bytes);
-                            } else {
-                                self.push_int_from_slice(
-                                    start,
-                                    end,
+                        match segment.value.as_ref() {
+                            Pattern::Int { int_value, .. }
+                                if details.size <= SAFE_INT_SEGMENT_MAX_SIZE =>
+                            {
+                                let bytes = bit_array_segment_int_value_to_bytes(
+                                    (*int_value).clone(),
+                                    BigInt::from(details.size),
                                     details.endianness,
-                                    details.is_signed,
-                                );
-                            }
-                        } else {
-                            self.push_float_from_slice(start, end, details.endianness);
-                        }
+                                )?;
 
-                        self.traverse_pattern(subject, &segment.value)?;
-                        self.pop();
-                        offset.increment(increment);
+                                for byte in bytes {
+                                    self.push_byte_at(offset.bytes);
+                                    self.push_equality_check(subject.clone(), docvec![byte]);
+                                    self.pop();
+                                    offset.increment(1);
+                                }
+                            }
+
+                            _ => {
+                                let start = offset.bytes;
+                                let increment = details.size / 8;
+                                let end = offset.bytes + increment;
+
+                                if segment.type_ == crate::type_::int() {
+                                    if details.size == 8 && !details.is_signed {
+                                        self.push_byte_at(offset.bytes);
+                                    } else {
+                                        self.push_int_from_slice(
+                                            start,
+                                            end,
+                                            details.endianness,
+                                            details.is_signed,
+                                        );
+                                    }
+                                } else {
+                                    self.push_float_from_slice(start, end, details.endianness);
+                                }
+
+                                self.traverse_pattern(subject, &segment.value)?;
+                                self.pop();
+                                offset.increment(increment);
+                            }
+                        }
                     } else {
                         match segment.options.as_slice() {
                             [Opt::Bytes { .. }] => {
@@ -840,7 +871,7 @@ pub struct CompiledPattern<'a> {
     pub assignments: Vec<Assignment<'a>>,
 }
 
-impl<'a> CompiledPattern<'a> {
+impl CompiledPattern<'_> {
     pub fn has_assignments(&self) -> bool {
         !self.assignments.is_empty()
     }
