@@ -111,7 +111,7 @@ fn go() {
 }
 
 #[test]
-fn sized() {
+fn sized_constant_value() {
     assert_js!(
         r#"
 fn go() {
@@ -122,7 +122,51 @@ fn go() {
 }
 
 #[test]
-fn sized_big_endian() {
+fn sized_dynamic_value() {
+    assert_js!(
+        r#"
+fn go(i: Int) {
+  <<i:64>>
+}
+"#,
+    );
+}
+
+#[test]
+fn sized_constant_value_positive_overflow() {
+    assert_js!(
+        r#"
+fn go() {
+  <<80_000:16>>
+}
+"#,
+    );
+}
+
+#[test]
+fn sized_constant_value_negative_overflow() {
+    assert_js!(
+        r#"
+fn go() {
+  <<-80_000:16>>
+}
+"#,
+    );
+}
+
+#[test]
+fn sized_constant_value_max_size_for_compile_time_evaluation() {
+    assert_js!(
+        r#"
+fn go() {
+  <<-1:48>>
+}
+"#,
+    );
+}
+
+#[test]
+fn sized_big_endian_constant_value() {
     assert_js!(
         r#"
 fn go() {
@@ -133,7 +177,18 @@ fn go() {
 }
 
 #[test]
-fn sized_little_endian() {
+fn sized_big_endian_dynamic_value() {
+    assert_js!(
+        r#"
+fn go(i: Int) {
+  <<i:16-big>>
+}
+"#,
+    );
+}
+
+#[test]
+fn sized_little_endian_constant_value() {
     assert_js!(
         r#"
 fn go() {
@@ -144,11 +199,33 @@ fn go() {
 }
 
 #[test]
-fn explicit_sized() {
+fn sized_little_endian_dynamic_value() {
+    assert_js!(
+        r#"
+fn go(i: Int) {
+  <<i:16-little>>
+}
+"#,
+    );
+}
+
+#[test]
+fn explicit_sized_constant_value() {
     assert_js!(
         r#"
 fn go() {
-  <<256:size(64)>>
+  <<256:size(32)>>
+}
+"#,
+    );
+}
+
+#[test]
+fn explicit_sized_dynamic_value() {
+    assert_js!(
+        r#"
+fn go(i: Int) {
+  <<i:size(32)>>
 }
 "#,
     );
@@ -297,6 +374,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_sized_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<1234:16, 123:8>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_unsigned() {
     assert_js!(
@@ -308,6 +396,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_unsigned_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<-2:unsigned>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_signed() {
     assert_js!(
@@ -319,6 +418,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_signed_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<-1:signed>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_sized_big_endian() {
     assert_js!(
@@ -330,6 +440,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_sized_big_endian_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<1234:16-big>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_sized_little_endian() {
     assert_js!(
@@ -341,6 +462,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_sized_little_endian_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<1234:16-little>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_sized_big_endian_unsigned() {
     assert_js!(
@@ -352,6 +484,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_sized_big_endian_unsigned_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<1234:16-big-unsigned>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_sized_big_endian_signed() {
     assert_js!(
@@ -363,6 +506,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_sized_big_endian_signed_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<1234:16-big-signed>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_sized_little_endian_unsigned() {
     assert_js!(
@@ -374,6 +528,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_sized_little_endian_unsigned_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<1234:16-little-unsigned>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_sized_little_endian_signed() {
     assert_js!(
@@ -385,6 +550,17 @@ fn go(x) {
     );
 }
 
+#[test]
+fn match_sized_little_endian_signed_constant_pattern() {
+    assert_js!(
+        r#"
+fn go(x) {
+  let assert <<1234:16-little-signed>> = x
+}
+"#,
+    );
+}
+
 #[test]
 fn match_dynamic_size_error() {
     assert_js_error!(
@@ -424,6 +600,17 @@ fn go(x) {
 fn match_sized_value() {
     assert_js!(
         r#"
+fn go(x) {
+  let assert <<i:16>> = x
+}
+"#,
+    );
+}
+
+#[test]
+fn match_sized_value_constant_pattern() {
+    assert_js!(
+        r#"
 fn go(x) {
   let assert <<258:16>> = x
 }
@@ -555,6 +742,7 @@ fn as_module_const() {
             "Gleam":utf8,
             4.2:float,
             4.2:32-float,
+            -1:64,
             <<
               <<1, 2, 3>>:bits,
               "Gleam":utf8,
@@ -569,7 +757,18 @@ fn as_module_const() {
 fn negative_size() {
     assert_js!(
         r#"
-fn go() {
+fn go(x: Int) {
+  <<x:size(-1)>>
+}
+"#,
+    );
+}
+
+#[test]
+fn negative_size_constant_value() {
+    assert_js!(
+        r#"
+fn go(x: Int) {
   <<1:size(-1)>>
 }
 "#,
