@@ -125,14 +125,6 @@ impl LanguageServerTestIO {
 }
 
 impl FileSystemReader for LanguageServerTestIO {
-    fn gleam_source_files(&self, dir: &Utf8Path) -> Vec<Utf8PathBuf> {
-        self.io.gleam_source_files(dir)
-    }
-
-    fn gleam_cache_files(&self, dir: &Utf8Path) -> Vec<Utf8PathBuf> {
-        self.io.gleam_cache_files(dir)
-    }
-
     fn read_dir(&self, path: &Utf8Path) -> Result<ReadDir> {
         self.io.read_dir(path)
     }
