@@ -1274,3 +1274,74 @@ pub fn main() {
         find_position_of("w: Wibble").under_char('b')
     );
 }
+
+#[test]
+fn hover_for_label_in_pattern() {
+    let code = "
+type Wibble {
+  Wibble(wibble: Int, wobble: Int)
+}
+
+pub fn main() {
+  let Wibble(wibble: _, wobble: _) = todo
+  todo
+}
+";
+
+    assert_hover!(
+        TestProject::for_source(code),
+        find_position_of("wibble: _").under_char('l')
+    );
+}
+
+#[test]
+fn hover_for_label_in_expression() {
+    let code = "
+fn add(wibble a, wobble b) {
+  a + b
+}
+
+pub fn main() {
+  add(wibble: 1, wobble: 2)
+}
+";
+
+    assert_hover!(
+        TestProject::for_source(code),
+        find_position_of("wibble:").under_char('i')
+    );
+}
+
+#[test]
+fn hover_for_pattern_in_use() {
+    let code = "
+type Wibble {
+  Wibble(Int, Float)
+}
+
+pub fn main() {
+  use Wibble(int, float) <- todo
+  todo
+}
+";
+
+    assert_hover!(
+        TestProject::for_source(code),
+        find_position_of("int").under_char('i')
+    );
+}
+
+#[test]
+fn hover_for_annotation_in_use() {
+    let code = "
+pub fn main() {
+  use something: Int <- todo
+  todo
+}
+";
+
+    assert_hover!(
+        TestProject::for_source(code),
+        find_position_of("Int").under_char('n')
+    );
+}
