@@ -66,6 +66,7 @@ pub mod config;
 pub mod dependency;
 pub mod diagnostic;
 pub mod docs;
+pub mod encryption;
 pub mod erlang;
 pub mod error;
 pub mod fix;
@@ -101,6 +102,13 @@ const GLEAM_CORE_PACKAGE_NAME: &str = "";
 const STDLIB_PACKAGE_NAME: &str = "gleam_stdlib";
 
 mod schema_capnp {
-    #![allow(dead_code, unused_qualifications)]
+    #![allow(
+        dead_code,
+        unused_qualifications,
+        clippy::all,
+        clippy::unwrap_used,
+        missing_debug_implementations,
+        missing_copy_implementations
+    )]
     include!("../generated/schema_capnp.rs");
 }
