@@ -26,7 +26,7 @@ use crate::{
     ast::{
         ArgNames, BitArraySegment, CallArg, Constant, DefinitionLocation, Pattern, Publicity,
         SrcSpan, TypedConstant, TypedExpr, TypedPattern, TypedPatternBitArraySegment,
-        TypedRecordUpdateArg, UntypedMultiPattern, UntypedPattern, UntypedRecordUpdateArg,
+        UntypedMultiPattern, UntypedPattern, UntypedRecordUpdateArg,
     },
     bit_array,
     build::{Origin, Target},
@@ -179,6 +179,7 @@ impl Type {
     pub fn tuple_types(&self) -> Option<Vec<Arc<Self>>> {
         match self {
             Self::Tuple { elems } => Some(elems.clone()),
+            Self::Var { type_, .. } => type_.borrow().tuple_types(),
             _ => None,
         }
     }
@@ -264,6 +265,16 @@ impl Type {
         }
     }
 
+    pub fn named_type_information(&self) -> Option<(EcoString, EcoString, Vec<Arc<Self>>)> {
+        match self {
+            Self::Named {
+                module, name, args, ..
+            } => Some((module.clone(), name.clone(), args.clone())),
+            Self::Var { type_ } => type_.borrow().named_type_information(),
+            _ => None,
+        }
+    }
+
     pub fn set_custom_type_variant(&mut self, index: u16) {
         match self {
             Type::Named {
@@ -924,6 +935,13 @@ impl TypeVar {
         }
     }
 
+    pub fn tuple_types(&self) -> Option<Vec<Arc<Type>>> {
+        match self {
+            Self::Link { type_ } => type_.tuple_types(),
+            Self::Unbound { .. } | Self::Generic { .. } => None,
+        }
+    }
+
     pub fn constructor_types(&self) -> Option<Vec<Arc<Type>>> {
         match self {
             Self::Link { type_ } => type_.constructor_types(),
@@ -1009,6 +1027,13 @@ impl TypeVar {
         }
     }
 
+    pub fn named_type_information(&self) -> Option<(EcoString, EcoString, Vec<Arc<Type>>)> {
+        match self {
+            Self::Link { type_ } => type_.named_type_information(),
+            Self::Unbound { .. } | Self::Generic { .. } => None,
+        }
+    }
+
     pub fn set_custom_type_variant(&mut self, index: u16) {
         match self {
             Self::Link { type_ } => Arc::make_mut(type_).set_custom_type_variant(index),
