@@ -137,8 +137,9 @@ pub enum EntityKind {
     ImportedValue,
     PrivateType,
     Variable {
-        /// How the variable could be rewritten to ignore it when unused
-        how_to_ignore: Option<EcoString>,
+        /// The origin of the variable so we know how it could be
+        /// rewritten to ignore it when unused.
+        origin: VariableOrigin,
     },
 }
 
@@ -147,7 +148,7 @@ pub struct ScopeResetData {
     local_values: im::HashMap<EcoString, ValueConstructor>,
 }
 
-impl<'a> Environment<'a> {
+impl Environment<'_> {
     pub fn in_new_scope<T, E>(
         &mut self,
         problems: &mut Problems,
@@ -349,7 +350,7 @@ impl<'a> Environment<'a> {
         module_alias: &Option<(EcoString, SrcSpan)>,
         name: &EcoString,
     ) -> Result<&TypeConstructor, UnknownTypeConstructorError> {
-        let t = match module_alias {
+        match module_alias {
             None => self
                 .module_types
                 .get(name)
@@ -368,19 +369,16 @@ impl<'a> Environment<'a> {
                 })?;
                 let _ = self.unused_modules.remove(module_name);
                 let _ = self.unused_module_aliases.remove(module_name);
-                module
-                    .types
-                    .get(name)
-                    .ok_or_else(|| UnknownTypeConstructorError::ModuleType {
+                module.get_public_type(name).ok_or_else(|| {
+                    UnknownTypeConstructorError::ModuleType {
                         name: name.clone(),
                         module_name: module.name.clone(),
                         type_constructors: module.public_type_names(),
                         imported_type_as_value: false,
-                    })
+                    }
+                })
             }
-        }?;
-
-        Ok(t)
+        }
     }
 
     fn unknown_type_hint(&self, type_name: &EcoString) -> UnknownTypeHint {
@@ -693,10 +691,7 @@ impl<'a> Environment<'a> {
                     location,
                 },
                 EntityKind::ImportedValue => Warning::UnusedImportedValue { name, location },
-                EntityKind::Variable { how_to_ignore } => Warning::UnusedVariable {
-                    location,
-                    how_to_ignore,
-                },
+                EntityKind::Variable { origin } => Warning::UnusedVariable { location, origin },
             };
 
             problems.warning(warning);
