@@ -201,6 +201,7 @@ pub enum Error {
         module_name: EcoString,
         value_constructors: Vec<EcoString>,
         type_with_same_name: bool,
+        context: ModuleValueUsageContext,
     },
 
     ModuleAliasUsedAsName {
@@ -561,6 +562,32 @@ pub enum Error {
         kind: Named,
         name: EcoString,
     },
+
+    /// Occers when all the variant types of a custom type are deprecated
+    ///
+    /// ```gleam
+    /// type Wibble {
+    ///     @deprecated("1")
+    ///     Wobble1
+    ///     @deprecated("1")
+    ///     Wobble1
+    /// }
+    /// ```
+    AllVariantsDeprecated {
+        location: SrcSpan,
+    },
+
+    /// Occers when any varient of a custom type is deprecated while
+    /// the custom type itself is deprecated
+    DeprecatedVariantOnDeprecatedType {
+        location: SrcSpan,
+    },
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum ModuleValueUsageContext {
+    UnqualifiedImport,
+    ModuleAccess,
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -619,6 +646,34 @@ impl Named {
     }
 }
 
+#[derive(Debug, Clone, PartialEq, Eq, serde::Serialize, serde::Deserialize)]
+/// The origin of a variable. Used to determine how it can be ignored when unused.
+pub enum VariableOrigin {
+    /// A variable that can be ignored by prefixing with an underscore, `_name`
+    Variable(EcoString),
+    /// A variable from label shorthand syntax, which can be ignored with an underscore: `label: _`
+    LabelShorthand(EcoString),
+    /// A variable from an assignment pattern, which can be ignored by removing `as name`,
+    AssignmentPattern,
+    /// A variable generated by the compiler. This should never need to be ignored
+    Generated,
+}
+
+impl VariableOrigin {
+    pub fn how_to_ignore(&self) -> Option<String> {
+        match self {
+            VariableOrigin::Variable(name) => {
+                Some(format!("You can ignore it with an underscore: `_{name}`."))
+            }
+            VariableOrigin::LabelShorthand(label) => Some(format!(
+                "You can ignore it with an underscore: `{label}: _`."
+            )),
+            VariableOrigin::AssignmentPattern => Some("You can safely remove it.".to_string()),
+            VariableOrigin::Generated => None,
+        }
+    }
+}
+
 #[derive(Debug, Eq, PartialEq, Clone, serde::Serialize, serde::Deserialize)]
 pub enum Warning {
     Todo {
@@ -687,8 +742,7 @@ pub enum Warning {
 
     UnusedVariable {
         location: SrcSpan,
-        /// how the variable could be rewritten to be ignored.
-        how_to_ignore: Option<EcoString>,
+        origin: VariableOrigin,
     },
 
     UnnecessaryDoubleIntNegation {
@@ -860,24 +914,36 @@ pub enum FeatureKind {
     AtInJavascriptModules,
     RecordUpdateVariantInference,
     RecordAccessVariantInference,
+    LetAssertWithMessage,
+    VariantWithDeprecatedAnnotation,
 }
 
 impl FeatureKind {
     pub fn required_version(&self) -> Version {
         match self {
-            FeatureKind::InternalAnnotation => Version::new(1, 1, 0),
-            FeatureKind::NestedTupleAccess => Version::new(1, 1, 0),
+            FeatureKind::InternalAnnotation | FeatureKind::NestedTupleAccess => {
+                Version::new(1, 1, 0)
+            }
+
             FeatureKind::AtInJavascriptModules => Version::new(1, 2, 0),
+
             FeatureKind::ArithmeticInGuards => Version::new(1, 3, 0),
-            FeatureKind::LabelShorthandSyntax => Version::new(1, 4, 0),
-            FeatureKind::ConstantStringConcatenation => Version::new(1, 4, 0),
+
+            FeatureKind::LabelShorthandSyntax | FeatureKind::ConstantStringConcatenation => {
+                Version::new(1, 4, 0)
+            }
+
             FeatureKind::UnannotatedUtf8StringSegment => Version::new(1, 5, 0),
+
             FeatureKind::RecordUpdateVariantInference
             | FeatureKind::RecordAccessVariantInference => Version::new(1, 6, 0),
+
+            FeatureKind::VariantWithDeprecatedAnnotation | FeatureKind::LetAssertWithMessage => {
+                Version::new(1, 7, 0)
+            }
         }
     }
 }
-
 #[derive(Debug, Eq, PartialEq, Clone, Copy, serde::Serialize, serde::Deserialize)]
 pub enum PanicPosition {
     /// When the unreachable part is a function argument, this means that one
@@ -972,7 +1038,9 @@ impl Error {
             }
             | Error::UseFnDoesntTakeCallback { location, .. }
             | Error::UseFnIncorrectArity { location, .. }
-            | Error::BadName { location, .. } => location.start,
+            | Error::BadName { location, .. }
+            | Error::AllVariantsDeprecated { location }
+            | Error::DeprecatedVariantOnDeprecatedType { location } => location.start,
             Error::UnknownLabels { unknown, .. } => {
                 unknown.iter().map(|(_, s)| s.start).min().unwrap_or(0)
             }
@@ -1101,6 +1169,7 @@ pub fn convert_get_value_constructor_error(
             module_name,
             value_constructors,
             type_with_same_name: imported_value_as_type,
+            context: ModuleValueUsageContext::ModuleAccess,
         },
     }
 }
@@ -1114,6 +1183,13 @@ pub enum UnsafeRecordUpdateReason {
         constructed_variant: EcoString,
         spread_variant: EcoString,
     },
+    IncompatibleFieldTypes {
+        constructed_variant: Arc<Type>,
+        record_variant: Arc<Type>,
+        expected_field_type: Arc<Type>,
+        record_field_type: Arc<Type>,
+        field_name: EcoString,
+    },
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
