@@ -210,7 +210,7 @@ impl Names {
             return NameContextInformation::Qualified(module, name.as_str());
         };
 
-        return NameContextInformation::Unimported(name.as_str());
+        NameContextInformation::Unimported(name.as_str())
     }
 
     /// Record a named value in this module.
@@ -246,6 +246,10 @@ impl Names {
 
         NameContextInformation::Unimported(name.as_str())
     }
+
+    pub fn is_imported(&self, module: &str) -> bool {
+        self.imported_modules.contains_key(module)
+    }
 }
 
 #[derive(Debug)]
@@ -318,6 +322,14 @@ impl<'a> Printer<'a> {
         buffer
     }
 
+    pub fn print_module(&self, module: &str) -> EcoString {
+        if let Some(module) = self.names.imported_modules.get(module) {
+            module.clone()
+        } else {
+            module.split("/").last().unwrap_or(module).into()
+        }
+    }
+
     pub fn print_type_without_aliases(&mut self, type_: &Type) -> EcoString {
         let mut buffer = EcoString::new();
         self.print(type_, &mut buffer, PrintMode::ExpandAliases);
