@@ -256,7 +256,7 @@ macro_rules! assert_warnings_with_imports {
             vec![
                 $(("thepackage", $name, $module_src)),*
             ],
-            $crate::build::Target::Erlang,
+            crate::build::Target::Erlang,
             None
         );
 
@@ -272,7 +272,7 @@ macro_rules! assert_warnings_with_imports {
 #[macro_export]
 macro_rules! assert_warning {
     ($src:expr) => {
-        let warning = $crate::type_::tests::get_printed_warnings($src, vec![], $crate::build::Target::Erlang, None);
+        let warning = $crate::type_::tests::get_printed_warnings($src, vec![], crate::build::Target::Erlang, None);
         assert!(!warning.is_empty());
         let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
@@ -309,7 +309,7 @@ macro_rules! assert_js_warning {
         let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![],
-            $crate::build::Target::JavaScript,
+            crate::build::Target::JavaScript,
             None,
         );
         assert!(!warning.is_empty());
@@ -324,7 +324,7 @@ macro_rules! assert_js_no_warnings {
         let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![],
-            $crate::build::Target::JavaScript,
+            crate::build::Target::JavaScript,
             None,
         );
         assert!(warning.is_empty());
@@ -337,7 +337,7 @@ macro_rules! assert_warnings_with_gleam_version {
         let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![],
-            $crate::build::Target::Erlang,
+            crate::build::Target::Erlang,
             Some($gleam_version),
         );
         assert!(!warning.is_empty());
@@ -349,7 +349,7 @@ macro_rules! assert_warnings_with_gleam_version {
 #[macro_export]
 macro_rules! assert_no_warnings {
     ($src:expr $(,)?) => {
-        let warnings = $crate::type_::tests::get_warnings($src, vec![], $crate::build::Target::Erlang, None);
+        let warnings = $crate::type_::tests::get_warnings($src, vec![], crate::build::Target::Erlang, None);
         assert_eq!(warnings, vec![]);
     };
     ($(($package:expr, $name:expr, $module_src:literal)),+, $src:expr $(,)?) => {
@@ -2989,3 +2989,53 @@ pub fn rebox(box) {
         ]
     );
 }
+
+#[test]
+// https://github.com/gleam-lang/gleam/issues/3861
+fn variant_inference_on_literal_record() {
+    assert_module_error!(
+        "
+pub type Wibble {
+  Wibble
+  Wobble
+}
+
+pub fn main() {
+  case Wibble, Wobble {
+    Wibble, Wibble -> todo
+  }
+}
+"
+    );
+}
+
+#[test]
+fn variant_inference_on_prelude_types() {
+    assert_module_infer!(
+        "
+pub fn main() {
+  let always_ok = Ok(10)
+  case always_ok {
+    Ok(1) -> 1
+    Ok(2) -> 3
+    _ -> panic
+  }
+}
+",
+        vec![("main", "fn() -> Int")]
+    );
+}
+
+#[test]
+fn private_types_not_available_in_other_modules() {
+    assert_with_module_error!(
+        ("wibble", "type Wibble"),
+        "
+import wibble
+
+type Wibble {
+  Wibble(wibble.Wibble)
+}
+"
+    );
+}
